#include 'common.ch'
#include 'directry.ch'

// Add #include '' to top of programs in a single directory
// released to public domain 8/7/96 Phil Barnett

Function Incl_Add()

local x
local aFiles
local cParms := DOSPARAM()
local cParm
local nParse
local cParmText := ''
local UParmText
local nHowMany
local lResult := .t.
local cFileSpec
local cThisFile

nParse := numtoken( cParms, ' ' )

// Parse the command line into it's parts
for x := 1 to nParse

   cParm := token( cParms, ' ', x )

   if x == 1
      cFileSpec := cParm
   else
      cParmText += ' ' + cParm
   endif

next

// trim off that beginning space
cParmText := ltrim( cParmText )

if empty( cFileSpec ) .or. empty( cParmText )
   ?
   ? 'INCL_ADD inserts text at the top of any text file'
   ? 'The standard use is to add header file references to source code'
   ?
   ? 'Note: It will not place the text if it is already in the file'
   ?
   ? 'Usage:'
   ?
   ? '    INCL_ADD <filespec> <text>'
   ?
   ? '    Where'
   ?
   ? '       <filespec> is any valid DOS file name. Wildcards OK.'
   ? '       <text>     is some text. (duh)'
   ?
   ? '    Processed files will have the extension .NEW'
   ?
   quit
endif

aFiles := directory( cFileSpec )

if empty( aFiles )
   ?
   ? 'No files found matching file spec: ' + cFileSpec
   ?
   quit
endif

UParmText := Upper( cParmText )

nHowmany := len( aFiles )

for x := 1 to nHowmany
   cThisFile := aFiles[ x, F_NAME  ]
   if !infile( cThisFile, cParmText )
      lResult := add2file( cThisFile, cParmText )
   endif
   if !lResult
      exit
   endif
next

If x <= nHowmany
   ? 'Processing loop failed on file: ' + aFiles[ x, 1 ]
Else
   ? 'All matching files processed as needed'
Endif

Return Nil


function infile( cFileName, cText )

local lResult := .f.
local lookfor := upper( cText )

if bopen( cFileName )
   do while !beof()
      if lookfor $ upper( bReadLine() )
         lResult := .t.
         exit
      endif
   enddo
   bclose()
endif

return lResult

function add2file( cFileName, cText )

local nInHandle := fopen( cFileName )
local nOutHandle := fcreate( rootname( cFileName ) + '.NEW' )
local nBuffSize := 16384
local cBuffer := space( nBuffSize )
local lResult := .f.
local nBytesRead

if nOutHandle > -1 .and. nInHandle > -1

   fwrite( nOutHandle, cText + chr( 13 ) + chr( 10 ) )

   do while .t.

      nBytesRead := fread( nInHandle, @cBuffer, nBuffSize )

      fwrite( nOutHandle, cBuffer, nBytesRead )

      if nBytesRead <> nBuffSize
         lResult := .t.
         exit
      endif

   enddo

endif

return lResult

function rootname( cFileName )

local nPointer := rat( '.', cFileName )
local lResult  := cFileName

if nPointer > 0
  lResult := left( cFileName, nPointer - 1 )
endif

return lResult

*: EOF: INCL_ADD.PRG
